package cd.casic.ci.common.pipeline.pojo.setting;

import cd.casic.ci.common.pipeline.pojo.PipelineAsCodeSettings;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.List;

import static cd.casic.ci.common.pipeline.utils.Constants.*;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.setting
 * @Project：ops-pro
 * @name：PipelineSetting
 * @Date：2025/03/26 14:18
 * @Filename：PipelineSetting
 */
@Data
@Accessors(chain = true)
@Schema(title = "流水线配置")
public class PipelineSetting {
    @Schema(title = "项目id", required = false, readOnly = true)
    private String projectId = "";
    @Schema(title = "流水线id", required = false, readOnly = true)
    private String pipelineId = "";
    // 流水线基础配置
    @Schema(title = "流水线名称", required = false)
    private String pipelineName = "";
    @Schema(title = "版本", required = false)
    private int version = 1;
    @Schema(title = "描述", required = false)
    private String desc = "";
    @Schema(title = "标签ID列表", required = false)
    private List<String> labels = List.of();
    @Schema(title = "标签名称列表（仅用于前端展示，不参与数据保存）", required = false)
    private List<String> labelNames = List.of();
    @Schema(title = "构建号生成规则", required = false)
    private String buildNumRule; // 构建号生成规则
    // 通知订阅相关配置
    //"被successSubscriptionList取代"
    @Schema(title = "订阅成功相关", required = false)
    private Subscription successSubscription = new Subscription();
    //"被failSubscriptionList取代"
    @Schema(title = "订阅失败相关", required = false)
    private Subscription failSubscription = new Subscription();
    @Schema(title = "订阅成功通知组", required = false)
    private List<Subscription> successSubscriptionList = null;
    @Schema(title = "订阅失败通知组", required = false)
    private List<Subscription> failSubscriptionList = null;
    // 运行控制、流水线禁用相关配置
    @Schema(title = "Lock 类型", required = false)
    private PipelineRunLockType runLockType = PipelineRunLockType.SINGLE_LOCK;
    @Schema(title = "最大排队时长", required = false)
    private int waitQueueTimeMinute = PIPELINE_SETTING_WAIT_QUEUE_TIME_MINUTE_DEFAULT;
    @Schema(title = "最大排队数量", required = false)
    private int maxQueueSize = PIPELINE_SETTING_MAX_QUEUE_SIZE_DEFAULT;
    @Schema(title = "并发时,设定的group", required = false)
    private String concurrencyGroup = PIPELINE_SETTING_CONCURRENCY_GROUP_DEFAULT;
    @Schema(title = "并发时,是否相同group取消正在执行的流水线", required = false)
    private boolean concurrencyCancelInProgress = false;
    @Schema(title = "并发构建数量限制", required = false)
    private Integer maxConRunningQueueSize = null; // MULTIPLE类型时，并发构建数量限制
    // 平台系统控制相关配置 —— 不作为生成版本的配置
    @Schema(title = "保存流水线编排的最大个数", required = false)
    private int maxPipelineResNum = PIPELINE_RES_NUM_MIN; // 保存流水线编排的最大个数
    @Schema(title = "重试时清理引擎变量表", required = false)
    private Boolean cleanVariablesWhenRetry = false;
    @Schema(title = "YAML流水线特殊配置", required = false)
    private PipelineAsCodeSettings pipelineAsCodeSettings;

    public static PipelineSetting defaultSetting(
            String projectId,
            String pipelineId,
            String pipelineName,
            Integer maxPipelineResNum,
            Subscription failSubscription,
            Boolean inheritedDialectSetting,
            String pipelineDialectSetting
    ) {

        return new PipelineSetting()
                .setProjectId(projectId)
                .setPipelineId(pipelineId)
                .setPipelineName(pipelineName)
                .setVersion(1)
                .setDesc(pipelineName)
                .setMaxPipelineResNum(maxPipelineResNum != null ? maxPipelineResNum : PIPELINE_RES_NUM_MIN)
                .setWaitQueueTimeMinute(PIPELINE_SETTING_WAIT_QUEUE_TIME_MINUTE_DEFAULT)
                .setMaxQueueSize(PIPELINE_SETTING_MAX_QUEUE_SIZE_DEFAULT)
                .setRunLockType(PipelineRunLockType.MULTIPLE)
                .setSuccessSubscription(null)
                .setFailSubscription(null)
                .setSuccessSubscriptionList(List.of())
                .setFailSubscriptionList(failSubscription != null ? List.of(failSubscription) : null)
                .setPipelineAsCodeSettings(PipelineAsCodeSettings.initDialect(
                        inheritedDialectSetting,
                        pipelineDialectSetting
                ));
    }

    // 校验流水线的通知设置是否为空，即用户未配置或使用默认配置
    public boolean notifySettingIsNull() {
        boolean res = true;
        if (successSubscriptionList != null && !successSubscriptionList.isEmpty() &&
                successSubscriptionList.stream().anyMatch(subscription -> !subscription.getTypes().isEmpty())) {
            res = false;
        }
        if (failSubscriptionList != null && !failSubscriptionList.isEmpty() &&
                failSubscriptionList.stream().anyMatch(subscription -> !subscription.getTypes().isEmpty())) {
            res = false;
        }
        if (successSubscription != null && !successSubscription.getTypes().isEmpty()) {
            res = false;
        }
        if (failSubscription != null && !failSubscription.getTypes().isEmpty()) {
            res = false;
        }
        return res;
    }

    // 校验流水线的并发组设置是否为空，即用户未配置或使用默认配置
    public boolean concurrencySettingIsNull() {
        return runLockType != PipelineRunLockType.GROUP_LOCK;
    }

    public void fixSubscriptions() {
        // 只有旧数据向新数据的更新，取消旧数据的保存
        if (successSubscriptionList == null && successSubscription != null) {
            successSubscriptionList = List.of(successSubscription);
        }
        successSubscription = successSubscriptionList != null ? successSubscriptionList.get(0) : null;
        if (failSubscriptionList == null && failSubscription != null) {
            failSubscriptionList = List.of(failSubscription);
        }
        failSubscription = failSubscriptionList != null ? failSubscriptionList.get(0) : null;
    }

    public void copySubscriptionSettings(PipelineSetting other) {
        this.successSubscription = other.getSuccessSubscription();
        this.successSubscriptionList = other.getSuccessSubscriptionList();
        this.failSubscription = other.getFailSubscription();
        this.failSubscriptionList = other.getFailSubscriptionList();
    }

    public void copyConcurrencyGroup(PipelineSetting other) {
        this.concurrencyGroup = other.getConcurrencyGroup();
        this.concurrencyCancelInProgress = other.concurrencyCancelInProgress;
        this.maxConRunningQueueSize = PIPELINE_SETTING_MAX_CON_QUEUE_SIZE_MAX;
    }

}